package cn.estsh.i3plus.pojo.base.dynamic;

import cn.estsh.i3plus.pojo.base.bean.BaseBean;
import com.google.common.base.CaseFormat;
import lombok.Getter;
import lombok.Setter;
import lombok.val;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description : 基础动态对象，包含方法、参数等
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2019-01-24 15:56
 * @Modify:
 **/
public class DynamicEntity extends BaseBean implements Serializable {

    public static final Logger LOGGER = LoggerFactory.getLogger(DynamicEntity.class);

    public String tableName;
    /*private String uri;
    private String method;
    private Object[] args;
    private Object result;
    private String operator;
    private String appName;*/

    public List<String> propertyList;

    public DynamicEntity(){
        try {
            this.setPropertyList(new ArrayList<>());
            initDynamic();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public DynamicEntity(String tableName){
        this.tableName = tableName;
    }

    public String getTableName() {
        return tableName;
    }

    public void setTableName(String tableName) {
        this.tableName = tableName;
    }

    /**
     * 初始化属性，以便动态加载
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws NoSuchMethodException
     */
    public void initDynamic() {
        Field[] fields = this.getClass().getDeclaredFields();
        Method setMethod = null;
        String setMethodName,propName;
        Object fieldVal = null;

        for(Field f : fields) {
            propName = f.getName().replace("$cglib_prop_", "");
            if(!"LOGGER".equals(propName) && !"propertyList".equals(propName) && !"dynProperty".equals(propName)) {
                this.getPropertyList().add(propName); //添加到属性list中
                setMethodName = "set" + propName.substring(0,1).toUpperCase() + propName.substring(1);
                f.setAccessible(true);
                try {
                    fieldVal = f.get(this);
                } catch (IllegalAccessException e) {
                    fieldVal = null;
                }

                if(fieldVal == null) {
                    if (f.getType() == Integer.class) {
                        fieldVal = 0;
                    } else if (f.getType() == Long.class) {
                        fieldVal = 0L;
                    } else if (f.getType() == Float.class) {
                        fieldVal = 0.0f;
                    } else if (f.getType() == Double.class) {
                        fieldVal = 0.0d;
                    } else if (f.getType() == String.class) {
                        fieldVal = "";
                    }else if (f.getType() == Character.class) {
                        fieldVal = "";
                    }else if (f.getType() == Boolean.class) {
                        fieldVal = true;
                    }else if (f.getType() == Byte.class) {
                        fieldVal = 0;
                    }else if (f.getType() == Date.class) {
                        fieldVal = new Date();
                    }else {
                        fieldVal = "";
                    }
                }

                try {
                    setMethod = this.getClass().getDeclaredMethod(setMethodName, new Class[]{f.getType()});
//                    System.out.println("Method Name:" + setMethod.getName() + "\t\t Value : " + fieldVal);
                    setMethod.invoke(this, fieldVal);
                } catch (ClassCastException e) {
                    e.printStackTrace();
                    LOGGER.error("ClassCastException ：{}", setMethodName, e);
                } catch (IllegalArgumentException e) {
                    e.printStackTrace();
                    LOGGER.error("IllegalArgumentException ：{}", setMethodName, e);
                } catch (NoSuchMethodException e) {
                    LOGGER.error("没有方法：{}", setMethodName, e);
                } catch (IllegalAccessException e) {
                    LOGGER.error("入参出错：{}:{}:{}", f, f.getType(), fieldVal, e);
                } catch (InvocationTargetException e) {
                    LOGGER.error("方法返回出错：{}", setMethodName, e);
                }catch (RuntimeException e) {
                    LOGGER.error("RuntimeException ：{}", setMethodName, e);
                }catch (Exception e) {
                    LOGGER.error("Exception ：{}", setMethodName, e);
                }
            }
        }
    }

    @Override
    public String toString() {
        String result = "{";

        Object fieldVal = null;
        Field[] fields = this.getClass().getDeclaredFields();
        String fieldName;
        for(Field f : fields) {
            fieldName = f.getName().replace("$cglib_prop_", "");
            if(!"LOGGER".equals(fieldName) && !"propertyList".equals(fieldName)) {
                f.setAccessible(true);
                fieldVal = new Object();
                try {
                    fieldVal = f.get(this);
                } catch (IllegalAccessException e) {
                    fieldVal = null;
                }

                if (fieldVal == null) {
                    if (f.getType() == Integer.class || f.getType() == Long.class) {
                        fieldVal = 0;
                    } else if (f.getType() == Float.class || f.getType() == Double.class) {
                        fieldVal = 0.0;
                    } else {
                        fieldVal = "";
                    }
                }

                result += "\"" + fieldName + "\":\"" + fieldVal + "\",";
            }
        }
        if(fields.length > 0) {
            result = result.substring(0,result.length()-1);
        }

        result += "}";

        return result;
    }


    public void setDynProperty(Map<String,Object> params) {
        if (params != null && params.size() > 0){
            params.forEach((k,v)->{{
                setDynProperty(k,v);
            }});
        }
    }

    /**
     * 根据属性名，设置属性值
     * //TODO  wei.peng 设置为空时会无效
     * @param propName
     * @param val
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws NoSuchMethodException
     */
    public void setDynProperty(String propName,Object val){
        //初始化set方法
        String setMethodName = "set" + CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, propName);
        //获取方法
        Method setMethod = null;
        try {
            setMethod = this.getClass().getDeclaredMethod(setMethodName, new Class[]{val.getClass()});
            setMethod.invoke(this, val);
        } catch (NoSuchMethodException e) {
            LOGGER.error("没有方法：{}",setMethodName,e);
        } catch (IllegalAccessException e) {
            LOGGER.error("入参出错：{}:{}",val,val.getClass(),e);
        } catch (InvocationTargetException e) {
            LOGGER.error("方法返回出错：{}",setMethodName,e);
        }
    }

    /**
     * 获取属性值
     * @param propName
     * @return
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     */
    public Object getDynProperty(String propName){
        //初始化set方法
        String setMethodName = "get" + propName.substring(0,1).toUpperCase() + propName.substring(1);

        try {
            //获取方法
            Method getMethod = this.getClass().getDeclaredMethod(setMethodName);
            //实现方法
            return getMethod.invoke(this);
        } catch (NoSuchMethodException e) {
            LOGGER.error("没有方法：{}:{}",setMethodName,propName,e);
        } catch (IllegalAccessException e) {
            LOGGER.error("没有方法：{}:{}",setMethodName,propName,e);
        } catch (InvocationTargetException e) {
            LOGGER.error("方法返回出错：{}",setMethodName,e);
        }
        return null;
    }

    public List<String> getPropertyList() {
        return propertyList;
    }

    public void setPropertyList(List<String> propertyList) {
        this.propertyList = propertyList;
    }
}
